/// SVDRP Client
/// 
/// Fhrt Kommandos auf dem VDR Server aus.
/// 
/// 15.10.2004	-	initial Release		-	Daniel Kirstenpfad
/// 
/// Einige Teile des Quelltextes hier wurden teilweise mit erheblichen Modifikationen 
/// aus dem Artikel "Retrieve Mail From a POP3 Server Using C#" von Agus Kurniawan bernommen.
/// abrufbar hier: http://www.codeproject.com/csharp/popapp.asp
/// 
namespace VDRGUI
{
	using System;
	using System.IO ;
	using System.Net;
	using System.Net.Sockets;

	public class SVDRPClient
	{
		#region Objektdefinitonen
		private TcpClient Server;
		private NetworkStream Datenstrom;
		private StreamReader  DatenstromLeser;
		private string Daten;
		private int VDR_Server_Port;
		private string VDR_Server;
		private byte[] szDaten;
		private string CRLF = "\r\n";	// Zeilenabschluss
		private InvalidOperationException letzte_Fehlermeldung;
		public bool connected;
		private bool error;
		#endregion

		#region Konstruktoren
		/// <summary>
		/// Konstruktor
		/// </summary>
		public SVDRPClient()
		{
		}

		/// <summary>
		/// Konstruktor des SVDRP Clients
		/// </summary>
		/// <param name="vdr_server">Hostname oder IP des VDR-Servers</param>
		/// <param name="port">Port-Nummer auf dem der VDR-Server luft</param>
		public SVDRPClient(string vdr_server,int port)
		{
			VDR_Server=vdr_server;
			VDR_Server_Port=port;
		}
		#endregion

		#region Hilfsmethoden
		/// <summary>
		/// Liefert die letzte Fehlermeldung zurck
		/// </summary>
		/// <returns>die Exception die aufgetreten ist als InvalidOperationException</returns>
		public InvalidOperationException getLastErrorMessage()
		{
			return letzte_Fehlermeldung;
		}

		/// <summary>
		/// Verbindung aufbauen; Achtung: Unbedingt mit QUIT() wieder Verbindung abbauen
		/// </summary>
		/// <param name="vdr_server">Hostname oder IP des VDR-Servers</param>
		/// <param name="port">Port-Nummer auf dem der VDR-Server luft</param>
		/// <returns>Text der vom VDR als Willkommen-Meldung geschickt wird, oder NULL fr Fehler</returns>
		public string connect (string vdr_server,int port)
		{
			VDR_Server=vdr_server; 
			return(connect());
		}

		/// <summary>
		/// Verbindung aufbauen; Achtung: Unbedingt mit QUIT() wieder Verbindung abbauen
		/// </summary>
		/// <returns>Text der vom VDR als Willkommen-Meldung geschickt wird, oder NULL fr Fehler</returns>
		public string connect()
		{
			// Verbindungs-Objekt erzeugen mit Adresse und Port
			Server = new TcpClient(VDR_Server,VDR_Server_Port);					
		
			try
			{
				// Initialisierung
				Datenstrom = Server.GetStream();
				DatenstromLeser = new StreamReader(Server.GetStream());

				// "Willkommens"-Meldung abrufen
				string temporary = DatenstromLeser.ReadLine();
				
				connected = true;
				
				return(temporary);
			}			
			catch(InvalidOperationException err)
			{
				connected = false;
				letzte_Fehlermeldung = err;
				return(null);
			}
		}

		/// <summary>
		/// Verbindung abbauen
		/// </summary>
		/// <returns>
		/// TRUE = fehlerfrei
		/// FALSE = fehlerhaft, es ist keine Verbindung aufgebaut gewesen
		/// </returns>
		private bool disconnect ()
		{
			if(connected)
			{
				// Verbindung abbauen
				Datenstrom.Close();
				DatenstromLeser.Close();
				connected = false;
			}
			else
			{
				return false;
			}
			return(true);
		}

		/// <summary>
		/// schickt ein Kommando zum VDR Server
		/// </summary>
		/// <param name="kommando">Das zu schickende Kommando</param>
		private void kommando_schicken(string kommando)
		{
			Daten = kommando + CRLF;
			szDaten = System.Text.Encoding.ASCII.GetBytes(Daten.ToCharArray());
			Datenstrom.Write(szDaten,0,szDaten.Length);
		}

		/// <summary>
		/// Liest eine einzelne Zeile der Antwort des Servers
		/// </summary>
		/// <returns>Die Antwort des Servers</returns>
		private string read_single_line_response()
		{
			string temp;
			try
			{
				temp = DatenstromLeser.ReadLine();
				check_response(temp);			
				return(temp);
			}
			catch(InvalidOperationException err)
			{
				letzte_Fehlermeldung = err;
				return(null);
			}

		}

		/// <summary>
		/// Liest die komplette, mglicherweise mehrzeilige Antwort vom Server
		/// </summary>
		/// <returns>Antwort des Servers</returns>
		private string read_multi_line_response()
		{
			string temp="";
			string szTemp;

			try
			{
				szTemp = DatenstromLeser.ReadLine();
				check_response(szTemp);				
				if(!error) 
				{
					// kein Fehler
					while(szTemp.StartsWith("250-"))
					{
						temp += szTemp+CRLF;
						szTemp = DatenstromLeser.ReadLine();
					}
					temp += szTemp+CRLF;
				}
				else
				{
					// Fehler
					temp=szTemp;
				}
				return(temp);
			}
			catch(InvalidOperationException err)
			{
				letzte_Fehlermeldung = err;
				return(null);
			}
		}

		/// <summary>
		/// berprft ob die Antwort des Servers ein Fehler darstellt oder fehlerfrei abgelaufen ist
		/// </summary>
		/// <param name="response">Antwort vom Server</param>
		private void check_response(string response)
		{
			if(response.StartsWith ("5"))
			{
				error=true;
			}
			else
			{
				//success
				error=false;
			}
		}
		#endregion
		
		#region integrierte SVDRP Kommandos
		
		/// <summary>
		/// Fordert den spezifischen Timer(programmierte Aufnahme) an
		/// </summary>
		/// <param name="Nummer">Nummer des Timers</param>
		/// <returns>Timer</returns>
		public string LSTT(int Nummer)
		{
			string temp="";
			kommando_schicken("LSTT "+Nummer.ToString());
			temp=read_multi_line_response();	
			return(temp);			
		}
		/// <summary>
		/// Fordert alle Timer(programmierte Aufnahmen) an 
		/// </summary>
		/// <returns>Timer Listing</returns>
		public string LSTT()
		{
			string temp="";
			kommando_schicken("LSTT");
			temp=read_multi_line_response();	
			return(temp);			
		}

		/// <summary>
		/// Fordert Informationen zur spezifischen Aufnahme an
		/// </summary>
		/// <param name="Nummer">Nummer der Aufnahme</param>
		/// <returns>Informationen zur Aufnahme</returns>
		public string LSTR(int Nummer)
		{
			string temp="";
			kommando_schicken("LSTR "+Nummer.ToString());
			temp=read_multi_line_response();	
			return(temp);			
		}
		/// <summary>
		/// Fordert Liste aller Aufnahmen an
		/// </summary>
		/// <returns>Aufnahmen Listing</returns>
		public string LSTR()
		{
			string temp="";
			kommando_schicken("LSTR");
			temp=read_multi_line_response();	
			return(temp);			
		}

		/// <summary>
		/// Liefert Informationen ber den Festplattenspeicher-Verbrauch (Total, Frei, Prozent). 
		/// </summary>
		/// <returns>Informationen ber Festplattenspeicher-Verbrauch (Total,Frei,Prozent) in einem String Array</returns>
		public string[] STAT_DISK()
		{
			string temp="";
			kommando_schicken("STAT disk");
			temp=read_single_line_response();
			
			return(temp.Split(' '));			
		}

		/// <summary>
		/// Logt sich aus dem VDR-Server aus
		/// </summary>
		/// <returns>Antwort vom Server</returns>
		public string QUIT()
		{
			string temp="";
			kommando_schicken("QUIT");
			temp=read_single_line_response();			
			return(temp);			
		}

		#endregion
	}

}
